%
% Calculo del tiempo medio entre falsas alarmas, ARL0, del algoritmo CUMSUM
% para detectar cambios bruscos en la media de seales i.i.d. Gaussianas. 
% Todos los parametros son conocidos salvo el instante del cambio.
%
% Hace simulacion de Montecarlo promediando el tiempo en que se produce la primera
% falsa alarma.  


% Datos:
mu0 = 0; % media antes del cambio
mu1 = 1; % media despues del cambio
vari = 1; % varianza
umbral = 5; % umbral de deteccion

% Numero de secuencias analizadas
Ns=1e5;

% Inicializa el vector de instantes de falsas alarmas
Md = zeros(Ns,1);

% Precalculo de parametros
C1=(mu0^2-mu1^2)/(2*vari);
C2=(mu1-mu0)/vari;

for ns=1:Ns
   
    % Primera muestra
    x = mu0+sqrt(vari)*randn;
    s = C1+C2*x;
    S = s;
    Smin = S;
    T = S;    
    if T > umbral % deteccion de salto en la primera muestra
        Md(ns) = 1;
    end
    
    % Siguientes muestras
    k=2;
    while Md(ns) ==0
        x = mu0+sqrt(vari)*randn;    
        s = C1+C2*x;
        S = S + s;
        Smin = min([Smin S]);
        T = S - Smin;
        if T > umbral
            Md(ns) = k;
        end
        k=k+1;
    end
    
    %disp(['realizacion: ',num2str(ns),'      Md=',num2str(Md(ns))])

end

ARL0=mean(Md);


disp(' ')
disp(['umbral=',num2str(umbral)])
disp(['ARL0 =',num2str(ARL0)])
