%
% Calculo del retardo medio de deteccion del cambio , ARL1, del algoritmo
% CUMSUM para detectar cambios bruscos en la media de seales 
% i.i.d. Gaussianas. Todos los parametros son conocidos salvo el instante del cambio.
%
% Hace simulacion de Montecarlo promediando el retardo de deteccion
% para muchas secuencias. 
%
% Cada secuencia se compone de un numero variable de muestras y se toma
% la decision considerando las ultimas N muestras.


% Datos:
mu0 = 0; % media antes del cambio
mu1 = 1; % media despues del cambio
vari = 1; % varianza
umbral = 5.0; % umbral de deteccion

% Instante de cambio. Debe ser suficientemente grande
M=400; 

% Numero de secuencias analizadas
Ns=1e7;

% Inicializa los valores de instantes de deteccion (alarma)
Md = zeros(Ns,1);

% Inicializa  las estimas de instante de cambio
Me = zeros(Ns,1);

% Precalculo de parametros
C1=(mu0^2-mu1^2)/(2*vari);
C2=(mu1-mu0)/vari;


% Ciclo a a las realizaciones
for ns=1:Ns
    
    % Genero M-1 muestras antes del cambio
    x = mu0+sqrt(vari)*randn;
    s = C1+C2*x;
    S = s;
    Smin=s;
    kSmin=1;
    for k=2:M-1
        x = mu0+sqrt(vari)*randn;
        s = C1+C2*x;
        S = s + S;
        if  S < Smin
            Smin = S;
            kSmin = k;
        end
    end

    % Muestras despues del cambio
    k = M;
    while Md(ns)==0
        x=mu1+sqrt(vari)*randn; 
        s = C1+C2*x;
        S = s + S;
        if  S < Smin
            Smin = S;
            kSmin=k;
        end
        T = S - Smin;
        if T > umbral
            Md(ns) = k;
            Me(ns) = kSmin+1;
        end
        k=k+1;
    end

    %disp(['realizacion: ',num2str(ns),'      retardo det.=',num2str(Md(ns)-M),'     error det.=',num2str(abs(Me(ns)-M))])
    
end

ARL1=mean(Md-M);

disp(' ')
disp(['umbral=',num2str(umbral)])
disp(['ARL1 =',num2str(ARL1)])

% Error cuadratico medio del estimador del instante de cambio
MSE = mean(abs(Me-M).^2);
disp(['MSE =',num2str(MSE)])
